'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createCollection;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Method = require('./errors/Method');

var _Method2 = _interopRequireDefault(_Method);

var _Collection = require('./errors/Collection');

var _Collection2 = _interopRequireDefault(_Collection);

var _isModel = require('./isModel');

var _isModel2 = _interopRequireDefault(_isModel);

var _Collection3 = require('./base/Collection');

var _Collection4 = _interopRequireDefault(_Collection3);

var _Event = require('./base/Event');

var _Event2 = _interopRequireDefault(_Event);

var _isEvent = require('./isEvent');

var _isEvent2 = _interopRequireDefault(_isEvent);

var _events = require('./mixins/events');

var _events2 = _interopRequireDefault(_events);

var _bubbleUpEvent = require('./utils/bubbleUpEvent');

var _bubbleUpEvent2 = _interopRequireDefault(_bubbleUpEvent);

var _wrapCustomMethod = require('./utils/wrapCustomMethod');

var _wrapCustomMethod2 = _interopRequireDefault(_wrapCustomMethod);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function createCollection(Model) {
  var methods = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var initializers = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var Collection = function (_BaseCollection) {
    _inherits(Collection, _BaseCollection);

    function Collection() {
      var givenModels = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

      _classCallCheck(this, Collection);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Collection).call(this, givenModels));

      var models = [];

      // others listening to this
      var listeners = {};

      (0, _events2.default)(_this, listeners);

      var bubbleUp = function bubbleUp(model, eventName) {
        return (0, _bubbleUpEvent2.default)(_this, model, eventName, function (ctx, m) {
          return [ctx.findIndex(m)];
        });
      };

      Object.defineProperty(_this, 'length', {
        get: function get() {
          return models.length;
        }
      });

      _this.at = function (n) {
        return models[n];
      };

      _this.push = function (model) {
        var _this2 = this;

        if (!(0, _isModel2.default)(model)) {
          throw new _Collection2.default('not a valid Model instance is being pushed');
        }

        if (!(model instanceof Model)) {
          throw new _Collection2.default('Model instance is not of the one Collection is expecting');
        }

        this.trigger('method:call', new _Event2.default({ path: ['push'] }));

        var result = models.push(model);
        var index = result - 1;
        this.trigger('change', new _Event2.default({ path: [index] }));
        this.trigger('method:change', new _Event2.default({ path: ['push'] }));

        var changeWatcher = bubbleUp(model, 'change');
        var methodCallWatcher = bubbleUp(model, 'method:call');
        var methodChangeWatcher = bubbleUp(model, 'method:change');

        model.on('destroy', function () {
          _this2.remove(model);
          changeWatcher();
          methodCallWatcher();
          methodChangeWatcher();
        });

        model.on('remove', function () {
          _this2.trigger('change');
          changeWatcher();
          methodCallWatcher();
          methodChangeWatcher();
        });

        return result;
      };

      // native array methods
      ['every', 'filter', 'find', 'forEach', 'includes', 'indexOf', 'map', 'reduce', 'some'].forEach(function (readOnlyMethod) {
        _this[readOnlyMethod] = function (fn) {
          for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          return models[readOnlyMethod].apply(models, [fn.bind(this)].concat(args));
        };
      });

      // lodash methods
      ['difference', 'findIndex', 'first', 'last', 'nth', 'take', 'takeRight'].forEach(function (lodashMethod) {
        _this[lodashMethod] = function () {
          for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
          }

          return _lodash2.default[lodashMethod].apply(_lodash2.default, [models].concat(args));
        };
      });

      _this.pop = function () {
        this.trigger('method:call', new _Event2.default({ path: ['pop'] }));
        var model = models.pop();

        this.trigger('change');
        this.trigger('method:change', new _Event2.default({ path: ['pop'] }));

        model.trigger('remove');

        return model;
      };

      _this.shift = function () {
        this.trigger('method:call', new _Event2.default({ path: ['shift'] }));
        var model = models.shift();

        this.trigger('change');
        this.trigger('method:change', new _Event2.default({ path: ['shift'] }));

        model.trigger('remove');

        return model;
      };

      _this.unshift = function (model) {
        var _this3 = this;

        if (!(0, _isModel2.default)(model)) {
          throw new _Collection2.default('not a valid Model instance is being pushed');
        }

        if (!(model instanceof Model)) {
          throw new _Collection2.default('Model instance is not of the one Collection is expecting');
        }

        this.trigger('method:call', new _Event2.default({ path: ['unshift'] }));
        var result = models.unshift(model);

        this.trigger('change', new _Event2.default({ path: [0] }));
        this.trigger('method:change', new _Event2.default({ path: ['unshift'] }));

        var changeWatcher = bubbleUp(model, 'change');
        var methodCallWatcher = bubbleUp(model, 'method:call');
        var methodChangeWatcher = bubbleUp(model, 'method:change');

        model.on('destroy', function () {
          _this3.remove(model);
          _this3.trigger('change');

          changeWatcher();
          methodCallWatcher();
          methodChangeWatcher();
        });

        return result;
      };

      _this.remove = function (model) {
        var index = this.findIndex(model);

        this.removeFrom(index);
      };

      _this.removeFrom = function (index) {
        var model = models[index];

        if (!model) {
          return;
        }

        this.trigger('method:call', new _Event2.default({ path: ['removeFrom'] }));
        models.splice(index, 1);
        model.destroy();
        this.trigger('change');
        this.trigger('method:change', new _Event2.default({ path: ['removeFrom'] }));
      };

      _this.destroy = function () {
        this.trigger('method:call', new _Event2.default({ path: ['destroy'] }));
        models.forEach(function (model) {
          model.destroy();
        });

        this.trigger('destroy');
        this.trigger('method:change', new _Event2.default({ path: ['destroy'] }));
        this.off();
      };

      _this.toJS = function () {
        return models.map(function (model) {
          return model.toJS();
        });
      };

      // methods
      _lodash2.default.each(methods, function (methodFunc, methodName) {
        if (typeof _this[methodName] !== 'undefined') {
          throw new _Method2.default('conflicting method name: ' + methodName);
        }

        _this[methodName] = (0, _wrapCustomMethod2.default)(_this, methodName, methodFunc);
      });

      // initialize
      givenModels.forEach(function (v) {
        if ((0, _isModel2.default)(v)) {
          _this.push(v);

          return;
        }

        var model = new Model(v);
        _this.push(model);
      });

      // initializers
      initializers.forEach(function (initializer) {
        initializer(_this);
      });
      return _this;
    }

    return Collection;
  }(_Collection4.default);

  return Collection;
}